
/*
 * Copyright (C) 2006 Zack Rusin <zack@kde.org>
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2008 Kenneth Rohde Christiansen
 * Copyright (C) 2008 Diego Gonzalez
 * Copyright (C) 2009-2010 ProFUSION embedded systems
 * Copyright (C) 2009-2010 Samsung Electronics
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 * Copyright (c) 2012 NINTENDO.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "ChromeClientWiiU.h"
#include <wtf/Platform.h> // PLATFORM(...)

#if ENABLE(DATABASE)
#include "DatabaseDetails.h"
#include "DatabaseTracker.h"
#endif
#include "FileChooser.h"
#include "FloatRect.h"
#include "FrameLoader.h"
#include "Frame.h"
#include "Page.h"
#include "Settings.h"
#include "FrameLoaderClientWiiU.h"
#include "HitTestResult.h"
#include "IntRect.h"
#include "KURL.h"
#include "NotImplemented.h"
#include "PlatformString.h"
#include "SecurityOrigin.h"
#include "PopupMenuWiiU.h"
#include "SearchPopupMenuWiiU.h"
#include "ViewportArguments.h"
#include "WindowFeatures.h"
#include <wtf/text/CString.h>

#include <PlatformOS.h>
using namespace LibPlatform;

#include "WiiUEventManager.h"

using namespace WebCore;

namespace WebCore {

ChromeClientWiiU::ChromeClientWiiU(WebView* view)
    : m_view(view)
{
}

ChromeClientWiiU::~ChromeClientWiiU()
{
}

void ChromeClientWiiU::chromeDestroyed()
{
    delete this;
}

void ChromeClientWiiU::focusedNodeChanged(Node*)
{
    notImplemented();
}

void ChromeClientWiiU::focusedFrameChanged(Frame*)
{
}

FloatRect ChromeClientWiiU::windowRect()
{
	if(!m_view) {
		return FloatRect(0, 0, 0, 0);
	}

	return FloatRect(0, 0, m_view->width(), m_view->height());
}

void ChromeClientWiiU::setWindowRect(const FloatRect& rect)
{
    notImplemented();
}

FloatRect ChromeClientWiiU::pageRect()
{
    notImplemented();
    return FloatRect();
}

float ChromeClientWiiU::scaleFactor()
{
    notImplemented();
    return 1.0;
}

void ChromeClientWiiU::focus()
{
    notImplemented();
}

void ChromeClientWiiU::unfocus()
{
    notImplemented();
}

Page* ChromeClientWiiU::createWindow(Frame*, const FrameLoadRequest& frameLoadRequest, const WindowFeatures& features, const NavigationAction&)
{
	ResourceRequest request = frameLoadRequest.resourceRequest();

	String nullTerminated(String(request.url().string()).charactersWithNullTermination());
	return m_view->open_window(nullTerminated.utf8().data());
}

void ChromeClientWiiU::show()
{
    notImplemented();
}

bool ChromeClientWiiU::canRunModal()
{
    notImplemented();
    return false;
}

void ChromeClientWiiU::runModal()
{
    notImplemented();
}

void ChromeClientWiiU::setToolbarsVisible(bool visible)
{
    notImplemented();
}

bool ChromeClientWiiU::toolbarsVisible()
{
    notImplemented();

    return true;
}

void ChromeClientWiiU::setStatusbarVisible(bool visible)
{
    notImplemented();
}

bool ChromeClientWiiU::statusbarVisible()
{
    notImplemented();
    return true;
}

void ChromeClientWiiU::setScrollbarsVisible(bool visible)
{
    notImplemented();
}

bool ChromeClientWiiU::scrollbarsVisible()
{
    notImplemented();
    return true;
}

void ChromeClientWiiU::setMenubarVisible(bool visible)
{
    notImplemented();
}

bool ChromeClientWiiU::menubarVisible()
{
    notImplemented();

    return true;
}

void ChromeClientWiiU::showPopup(PopupMenuWiiU *popup, const IntRect& rect)
{
    m_view->setPopup(popup);
    m_view->setPopupRect(rect);
}

bool ChromeClientWiiU::hidePopup()
{
    m_view->setPopup(0);
    return true;
}

void ChromeClientWiiU::setResizable(bool)
{
    notImplemented();
}

void ChromeClientWiiU::closeWindowSoon()
{
    notImplemented();
}

bool ChromeClientWiiU::canTakeFocus(FocusDirection)
{
    notImplemented();
    // This is called when cycling through links/focusable objects and we
    // reach the last focusable object.
    return false;
}

void ChromeClientWiiU::takeFocus(FocusDirection)
{
    unfocus();
}

bool ChromeClientWiiU::canRunBeforeUnloadConfirmPanel()
{
    return true;
}

bool ChromeClientWiiU::runBeforeUnloadConfirmPanel(const String& message, Frame* frame)
{
    return runJavaScriptConfirm(frame, message);
}

void ChromeClientWiiU::addMessageToConsole(MessageSource, MessageType, MessageLevel, const String& message,
                                          unsigned int lineNumber, const String& sourceID)
{
	m_view->console_message(message.utf8().data(), lineNumber, sourceID.utf8().data());
}

void ChromeClientWiiU::runJavaScriptAlert(Frame* frame, const String& message)
{
	//message may or may not have a null terminator, so append one
	String modMessage(message);
	modMessage.append(L'\0');
	m_view->js_alert(message.utf8().data(), modMessage.characters());
}

bool ChromeClientWiiU::runJavaScriptConfirm(Frame* frame, const String& message)
{
	return m_view->js_confirm(message.utf8().data());
}

bool ChromeClientWiiU::runJavaScriptPrompt(Frame* frame, const String& message, const String& defaultValue, String& result)
{
	bool ret;
	char *res = NULL;

	ret = m_view->js_prompt(message.utf8().data(), defaultValue.utf8().data(), &res);
	if (res == NULL) {
		result = String("");
	} else {
		result = String(res);
		free(res);
	}
	return ret;
}

void ChromeClientWiiU::setStatusbarText(const String& string)
{
    m_view->status(WEBVIEW_STATUS, string.utf8().data());
}

bool ChromeClientWiiU::shouldInterruptJavaScript()
{
    return false;
}

KeyboardUIMode ChromeClientWiiU::keyboardUIMode()
{
    return KeyboardAccessTabsToLinks;
}

IntRect ChromeClientWiiU::windowResizerRect() const
{
    notImplemented();
    return IntRect();
}

void ChromeClientWiiU::contentsSizeChanged(Frame* frame, const IntSize& size) const
{
    notImplemented();
}

IntRect ChromeClientWiiU::windowToScreen(const IntRect& rect) const
{
    notImplemented();
    return rect;
}

IntPoint ChromeClientWiiU::screenToWindow(const IntPoint& point) const
{
    notImplemented();
    return point;
}

PlatformPageClient ChromeClientWiiU::platformPageClient() const
{
    return (PlatformPageClient)m_view;
}

void ChromeClientWiiU::scrollbarsModeDidChange() const
{
}

void ChromeClientWiiU::mouseDidMoveOverElement(const HitTestResult& hit, unsigned modifierFlags)
{
    // FIXME, compare with old link, look at Qt impl.
    bool isLink = hit.isLiveLink();
    if (isLink) {
        KURL url = hit.absoluteLinkURL();
        if (!url.isEmpty() && url != m_hoveredLinkURL) {
            m_hoveredLinkURL = url;
        }
    } else if (!isLink && !m_hoveredLinkURL.isEmpty()) {
        m_hoveredLinkURL = KURL();
    }
}

void ChromeClientWiiU::setToolTip(const String& toolTip, TextDirection)
{
    notImplemented();
//    OS::Report("TOOLTIP: %s\n", toolTip.utf8().data());
}

void ChromeClientWiiU::print(Frame* frame)
{
    notImplemented();
}

#if ENABLE(OFFLINE_WEB_APPLICATIONS)
void ChromeClientWiiU::reachedMaxAppCacheSize(int64_t spaceNeeded)
{
    // FIXME: Free some space.
    notImplemented();
}

void ChromeClientWiiU::reachedApplicationCacheOriginQuota(SecurityOrigin *origin,  int64_t totalSpaceNeeded)
{
    notImplemented();
}
#endif

#if ENABLE(DATABASE)
void ChromeClientWiiU::exceededDatabaseQuota(Frame* frame, const String& databaseName)
{
	Page *page = m_view->getPage();
	if(page) {
		const unsigned long long defaultQuota = page->settings()->sessionStorageQuota();
		if (!DatabaseTracker::tracker().hasEntryForOrigin(frame->document()->securityOrigin()))
			DatabaseTracker::tracker().setQuota(frame->document()->securityOrigin(), defaultQuota);
	}
}
#endif

#if ENABLE(NOTIFICATIONS)
NotificationPresenter* ChromeClientWiiU::notificationPresenter() const
{
	notImplemented();
    return 0;
}
#endif

#if ENABLE(REQUEST_ANIMATION_FRAME)
void ChromeClientWiiU::scheduleAnimation()
{ 
    m_view->scheduleAnimation();
}
#endif

void ChromeClientWiiU::runOpenPanel(Frame* frame, PassRefPtr<FileChooser> prpFileChooser)
{
    notImplemented();
}

void ChromeClientWiiU::formStateDidChange(const Node*)
{
    notImplemented();
}

void ChromeClientWiiU::setCursor(const Cursor &cursor)
{
    m_view->setCursor(&cursor);
}

void ChromeClientWiiU::setCursorHiddenUntilMouseMoves(bool)
{
    notImplemented(); 
}

void ChromeClientWiiU::requestGeolocationPermissionForFrame(Frame*, Geolocation*)
{
    // See the comment in WebCore/page/ChromeClient.h
    notImplemented();
}

void ChromeClientWiiU::cancelGeolocationPermissionRequestForFrame(Frame*, Geolocation*)
{
    notImplemented();
}

void ChromeClientWiiU::cancelGeolocationPermissionForFrame(Frame*, Geolocation*)
{
    notImplemented();
}

void ChromeClientWiiU::invalidateContents(const IntRect& updateRect, bool immediate)
{
    notImplemented();
    OS::Report("invalidateContents called!\n");
}

void ChromeClientWiiU::invalidateWindow(const IntRect& updateRect, bool immediate)
{
    invalidateContentsAndWindow(updateRect, immediate);
}

void ChromeClientWiiU::invalidateContentsAndWindow(const IntRect& updateRect, bool immediate)
{ 
    PlatformIntRect r;
 
    if (m_view->redrawPending()) {
        //merge dirty rects
        m_view->getDirtyRect(r);
        IntRect other(r.x, r.y, r.width, r.height);
        other.unite(updateRect);
        r.x = other.x();
        r.y = other.y();
        r.width = other.width();
        r.height = other.height();
        m_view->setDirtyRect(r);
        m_view->setRootLayerNeedsDisplay();
    } else {
    	WiiURedrawEvent* redrawEvent = new WiiURedrawEvent((WebViewHandle_t)m_view);
        r.x = updateRect.x();
        r.y = updateRect.y();
        r.width = updateRect.width();
        r.height = updateRect.height();
        m_view->setDirtyRect(r);
        m_view->setRedrawPending();
        m_view->setRootLayerNeedsDisplay();

        WebCore::WiiUEventManager::instance()->putEvent(redrawEvent);
    }
}

void ChromeClientWiiU::invalidateContentsForSlowScroll(const IntRect& updateRect, bool immediate)
{
    invalidateContentsAndWindow(updateRect, immediate);
}

void ChromeClientWiiU::scroll(const IntSize& scrollDelta, const IntRect& rectToScroll, const IntRect& clipRect)
{
    notImplemented();
}

void ChromeClientWiiU::cancelGeolocationPermissionRequestForFrame(Frame*)
{
    notImplemented();
}

void ChromeClientWiiU::iconForFiles(const Vector<String, 0u>&, PassRefPtr<FileChooser>)
{
    notImplemented();
}

void ChromeClientWiiU::loadIconForFiles(const Vector<String>&, FileIconLoader *)
{
    notImplemented();
}

bool ChromeClientWiiU::selectItemWritingDirectionIsNatural()
{
    return true;
}

bool ChromeClientWiiU::selectItemAlignmentFollowsMenuWritingDirection()
{
    return false;
}

void ChromeClientWiiU::dispatchViewportDataDidChange(const ViewportArguments& arguments) const
{

}

PassRefPtr<PopupMenu> ChromeClientWiiU::createPopupMenu(PopupMenuClient* client) const
{
    PopupMenuWiiU *menu = new PopupMenuWiiU(client);
    m_view->setPopup(menu); 
    return adoptRef(menu);
}

PassRefPtr<SearchPopupMenu> ChromeClientWiiU::createSearchPopupMenu(PopupMenuClient* client) const
{
    return adoptRef(new SearchPopupMenuWiiU(client));
}

#if ENABLE(TOUCH_EVENTS)
void ChromeClientWiiU::needTouchEvents(bool need)
{
	notImplemented();
}
#endif

#if USE(ACCELERATED_COMPOSITING)

void ChromeClientWiiU::attachRootGraphicsLayer(Frame *f, GraphicsLayer *l)
{
    m_view->setRootGraphicsLayer(l);
}

void ChromeClientWiiU::setNeedsOneShotDrawingSynchronization()
{
    scheduleCompositingLayerSync();
}

void ChromeClientWiiU::scheduleCompositingLayerSync()
{ 
    m_view->setRootLayerNeedsDisplay();

    PlatformIntRect r;
    r.x = 0;
    r.y = 0;
    r.width = m_view->width();
    r.height = m_view->height();
 
    if (m_view->redrawPending()) {
        m_view->setDirtyRect(r);
    } else {
    	WiiURedrawEvent* redrawEvent = new WiiURedrawEvent((WebViewHandle_t)m_view);
        m_view->setDirtyRect(r);
        m_view->setRedrawPending();

        WebCore::WiiUEventManager::instance()->putEvent(redrawEvent);
    } 
}

#endif



}
