#include "config.h"
#include "JSExtensionData.h"
#include "JSSystemEventDispatcher.h"
#include "JSContextRef.h"

#include <PlatformOS.h>
using namespace LibPlatform;

JSExtensionEventListener::JSExtensionEventListener(std::string listener_name)
	:JSEventListener(listener_name)
{

}

void JSExtensionEventListener::AddListenerCallback(JSEventListenerData* callback)
{	
	// Dont allow duplicate listener callbacks
	for(signed i = 0; i < listener_callbacks.size(); ++i) {
		if(listener_callbacks[i]->function == callback->function && 
		   listener_callbacks[i]->scopeObj == callback->scopeObj &&
		   (callback->eventName.compare(listener_callbacks[i]->eventName) == 0)) {
		   	OS::Report("Duplicate Event Listeners! %s\n", callback->eventName.c_str());
			return;
		}
	}

	//This is called to protect potentially anonymous functions
	if(callback->function)
	{
		JSValueProtect(callback->functionContext, callback->function);
	}
	if(callback->scopeObj)
	{
		JSValueProtect(callback->functionContext, callback->scopeObj);
	}
	listener_callbacks.push_back(callback);
}

void JSExtensionEventListener::RemoveListenerCallback(JSEventListenerData callback)
{
	for(int i = 0; i < listener_callbacks.size(); ++i) {

		if(callback.eventName.compare(listener_callbacks[i]->eventName) == 0)
		{
			if(callback.function == listener_callbacks[i]->function)
			{
				if(callback.scopeObj != NULL)
				{
					if(callback.scopeObj == listener_callbacks[i]->scopeObj)
					{
						if(listener_callbacks[i]->function)
						{
							JSValueUnprotect(listener_callbacks[i]->functionContext, listener_callbacks[i]->function);
						}
						if(listener_callbacks[i]->scopeObj)
						{
							JSValueUnprotect(listener_callbacks[i]->functionContext, listener_callbacks[i]->scopeObj);
						}
						delete listener_callbacks[i];
						listener_callbacks.erase(listener_callbacks.begin()+i);
						return;
					}
				}
				else
				{
					if(listener_callbacks[i]->function)
					{
						JSValueUnprotect(listener_callbacks[i]->functionContext, listener_callbacks[i]->function);
					}
					if(listener_callbacks[i]->scopeObj)
					{
						JSValueUnprotect(listener_callbacks[i]->functionContext, listener_callbacks[i]->scopeObj);
					}
					delete listener_callbacks[i];
					listener_callbacks.erase(listener_callbacks.begin()+i);
					return;
				}
			}
		}
	}
}

bool JSExtensionEventListener::HasListenerCallback(std::string callbackEventName)
{
	for(int i = 0; i < listener_callbacks.size(); ++i) {
		if(listener_callbacks[i] != NULL) {
			if(listener_callbacks[i]->eventName == callbackEventName) {
				return true;
			}
		}
	}
	return false;
}

void JSExtensionEventListener::ClearListenerCallbacks(void)
{
	for(int i = 0; i < listener_callbacks.size(); ++i) {
		if(listener_callbacks[i]->function)
		{
			JSValueUnprotect(listener_callbacks[i]->functionContext, listener_callbacks[i]->function);
		}
		if(listener_callbacks[i]->scopeObj)
		{
			JSValueUnprotect(listener_callbacks[i]->functionContext, listener_callbacks[i]->scopeObj);
		}
		delete listener_callbacks[i];
	}

	listener_callbacks.clear();
}