/*---------------------------------------------------------------------------*

  Copyright 2012 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/


#ifndef __WIIUEVENTS_H_INCLUDED
#define __WIIUEVENTS_H_INCLUDED

#include <inttypes.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Events
 */

enum {
	WIIUEVENTS_BUTTON_NONE,
	WIIUEVENTS_LEFT,
	WIIUEVENTS_MIDDLE,
	WIIUEVENTS_RIGHT,
};

typedef struct {
	int dummy; // This is here because the compiler doesn't like empty structures.  It can be removed when we flush this out
} WiiUEventExposeEvent;

typedef struct {
	int dummy; // This is here because the compiler doesn't like empty structures.  It can be removed when we flush this out
} WiiUEventResizeEvent;

typedef struct {
	int dummy; // This is here because the compiler doesn't like empty structures.  It can be removed when we flush this out
} WiiUEventQuitEvent;

typedef struct {
	int dummy; // This is here because the compiler doesn't like empty structures.  It can be removed when we flush this out
} WiiUEventUserEvent;

typedef struct {
	int state;
	uint8_t scancode;                 /**< hardware specific scancode */
	int 	sym;                     /**< SDL virtual keysym */
	int 	mod;                     /**< current key modifiers */
	uint16_t unicode;                 /**< translated character */
} WiiUEventKeyboardEvent;

typedef struct {
	int x;
	int y;
	int button;
	int state;
} WiiUEventMouseEvent;

typedef struct {
	int x;
	int y;
	int button;
	int state;
} WiiUEventWheelMouseEvent;

typedef struct {
	int dummy; // This is here because the compiler doesn't like empty structures.  It can be removed when we flush this out
} WiiUEventCursor;

typedef struct {
	int dummy; // This is here because the compiler doesn't like empty structures.  It can be removed when we flush this out
} WiiUEventWidget;

typedef struct {
	int x;
	int y;
} WiiUEventPoint;

typedef struct {
	int x;
	int y;
	int x2;
	int y2;
	int w;
	int h;
} WiiUEventRect;

typedef struct {
	int (*func)(void *);
	void *data;
	int msec;
} WiiUEventTimer;

enum {
	WIIUEVENT_CALLBACK_CANCEL = 1,
};

// function prototypes
void 		wiiuevent_check_timers();
int 		wiiuevent_get_time();
WiiUEventTimer * 	wiiuevent_timer_add(int t, int (*timeoutfunc)(void *), int m);
int 		wiiuevent_timer_remove(WiiUEventTimer *);

#ifdef __cplusplus
}
#endif

#endif

